<?php

namespace Drupal\pagedesigner_parts\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\node\Entity\Node;
use Drupal\pagedesigner\Service\Renderer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a pagedesigner part block.
 *
 * This block can be used to display the content of a page anywhere
 * in the block layout.
 *
 * @Block(
 *   id = "pagedesigner_part",
 *   admin_label = @Translation("Pagedesigner part"),
 * )
 */
class PagedesignerPart extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The node to render.
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $node = NULL;

  /**
   * The pagedesigner renderer service.
   *
   * @var \Drupal\pagedesigner\Service\Renderer
   */
  protected $renderer = NULL;

  /**
   * {@inheritDoc}
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values
   *   keyed by configuration option name.
   *   The special key 'context' may be used to initialize the defined contexts
   *   by setting it to an array of context values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\pagedesigner\Service\Renderer $renderer
   *   The pagedesigner renderer service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Renderer $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->renderer = $renderer;
  }

  /**
   * {@inheritDoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->get('pagedesigner.service.renderer')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function build() {
    // Create build array with defaults.
    $build = [];

    // Get block configuration.
    $config = $this->getConfiguration();
    $assignedNode = -1;

    // Check if assigned node id is avaiable.
    if (isset($config['assignedNode'])) {
      $assignedNode = $config['assignedNode'];
      $nodeId = \Drupal::routeMatch()->getRawParameter('node');

      // Only render when not on assigned node.
      if ($nodeId != $assignedNode) {
        $this->node = Node::load($assignedNode);

        // Check if the node is available.
        if ($this->node != NULL) {

          $language = \Drupal::service('language_manager')->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
          if ($this->node->hasTranslation($language)) {
            $this->node = $this->node->getTranslation($language);
            $container = $this->node->field_pagedesigner_content->entity;
            if ($container != NULL && $container->hasTranslation($language)) {
              $container = $container->getTranslation($language);

              if (!\Drupal::currentUser()->isAnonymous()) {
                // Render node for logged in users.
                $this->renderer->render($container, $this->node);
              }
              else {
                // Render node for anonymous users (public revisions only).
                $this->renderer->renderForPublic($container, $this->node);
              }

              // Get markup and additional css.
              $build = $this->renderer->getOutput();
            }
          }
        }
      }
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $node = NULL;
    $assignedNode = -1;
    if (isset($this->configuration['assignedNode'])) {
      $assignedNode = $this->configuration['assignedNode'];
      $node = Node::load($assignedNode);
    }
    $form['assignedNode'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Assigned part'),
      '#target_type' => 'node',
      '#selection_settings' => ['target_bundles' => ['pagedesigner_part']],
      '#tags' => TRUE,
      '#size' => 30,
      '#maxlength' => 1024,
      '#default_value' => $node,
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['assignedNode'] = $form_state->getValue('assignedNode')[0]["target_id"];
  }

}
